/*****************************************************************************************/
/*** SAS User Group International - Japan 2015                                         ***/
/*** Title: Analysis Results Metadata (ARM) Creation with SAS and Excel Metadata File  ***/
/*** File: ARM_Gen_SUGI_2015.sas                                                       ***/
/*** Author: Yohei Takanami (Takeda Pharmaceutical Co., Ltd.)                          ***/
/*****************************************************************************************/

/*
  <<Macro Parameters>>
    _CPATH: The path of current directory
    _MPATH: The path of Excel Metadata file
    _STUDY: Compound/Study name
*/

*** Set values to Macro Parameters ;
%let _CPATH = C:\temp\SUGI_2015\files ;
%let _MPATH = C:\temp\SUGI_2015\ARM_METADATA_SUGI_2015.xls ;
%let _STUDY = DRUG-XXX/STUDY-YYY ;

*** Libname for Metadata Excel Sheet with Excel engine ;
libname META excel "&_MPATH." SCAN_TEXTSIZE=YES ;
options mprint symbolgen mlogic source source2 linesize=200 ;

********** _CR_ARM Macro for creating ARM ********** ;
%macro _CR_ARM ;
   
   *** Read Analysis Reaults Metadata ;
   data RESULTS_METADATA ;
      set META.'Analysis Results Metadata$'n ;
      RESNO = _N_ ;
      if ID ne "" ;
   run ;

   data ARESM_WHR ;
      set META.'AResM Where$'n ;
      if WhereClauseOID ne "" ;
      WHRNO = _N_ ;
   run ;
   
   *** Read Documents information ;
   data DOCUMENTS ;
      set META.'Documents$'n ;
      DOCNO = _N_ ;
      if ID ne "" ;
   run ;
   ********** Analysis Results Metadata (V1.0) ********** ;
   filename _AR      "&_CPATH.\temp\_AR.txt" ;
   filename _ANPGMLF "&_CPATH.\temp\_ARPGM.txt" ;
   filename _ARWHR   "&_CPATH.\temp\_ARWHR.txt" ;

   proc sort data=RESULTS_METADATA ;
      by RESNO ID ;
   run ;

   *** def:leaf for Program files used in AResM ;
   data _null_ ;
      set RESULTS_METADATA end=_EOF ;
      file _ANPGMLF ;
      if ProgramID ne "" then do ;
         put '      <def:leaf ID="LF.' ProgramID + (-1) '" xlink:href="' href + (-1) '">' ;
         put '        <def:title>' Title + (-1) '</def:title>' ;
         put '      </def:leaf>' ;
      end ;
   run ;

   ********** Whereclause Definition for AResM ********** ;
   proc sort data=ARESM_WHR ;
      by Analysis_ID WhereClauseOID WHRNO ;
   run ;

   data _null_ ;
      set ARESM_WHR end=_EOF ;
      by Analysis_ID WhereClauseOID WHRNO ;
      file _ARWHR ;

      if first.WhereClauseOID then put '      <def:WhereClauseDef OID="WC.' WhereClauseOID + (-1) '">' ;
      put '        <RangeCheck Comparator="' Comparator + (-1) '" SoftHard="Soft" def:ItemOID="IT.' Dataset + (-1) '.' Variable + (-1) '">' ;
      VAL_C = count(Value, ",") + 1 ;
      do I = 1 to VAL_C ;
         VAL = scan(Value, I, ",") ;
         put '          <CheckValue>' VAL + (-1) '</CheckValue>' ;
      end ;
      put '        </RangeCheck>' ;

      if last.WhereClauseOID then put '      </def:WhereClauseDef>' ;
   run ;

   *** Main logic for output AResM ;
   proc sort data=RESULTS_METADATA ;
      by Analysis_ID ;
   run ;

   data _AResM ;
      merge RESULTS_METADATA ARESM_WHR ;
      by Analysis_ID ;
   run ;

   proc sort data=_AResM ;
      where ID ne "" ;
      by ID Analysis_ID Dataset WhereClauseOID ;
   run ;

   data _null_;
      set _AResM end=_EOF ;
      by ID Analysis_ID Dataset WhereClauseOID ;
      file _AR lrecl=1000 ; 

      ProgrammingCode = htmlencode(ProgrammingCode) ;

      if _N_ = 1 then put '      <arm:AnalysisResultDisplays>' ;

      if first.ID then do ;
         put '        <arm:ResultDisplay OID="RD.' ID + (-1) '" ' 'Name="' Name + (-1) '">' ; 
         put '          <Description>' ;
         put '            <TranslatedText xml:lang="en">' ResultDisplay + (-1) '</TranslatedText>' ;
         put '          </Description>' ;
      end ;

      if first.Analysis_ID then do ;
         put '          <arm:AnalysisResult OID="AR.'          Analysis_ID   + (-1) '"' ;
         put '                              ParameterOID="IT.' Parameter_OID + (-1) '"' ;
         put '                              AnalysisReason="'  Reason        + (-1) '"' ;
         put '                              AnalysisPurpose="' Purpose       + (-1) '">' ;
         put '            <Description>' ;
         put '              <TranslatedText xml:lang="en">' Description + (-1) '</TranslatedText>' ;
         put '            </Description>' ;
         put '            <arm:AnalysisDatasets>' ;
      end ;

      if first.Dataset then do ;
         put '              <arm:AnalysisDataset ItemGroupOID="IG.' Dataset + (-1) '" >' ;
         put '                <def:WhereClauseRef WhereClauseOID="WC.' WhereClauseOID   + (-1) '"/>' ;
         if Analysis_Variables ne "" then do ;
            _NVAL = count(Analysis_Variables,",") + 1 ;
            do I = 1 to _NVAL ;
               VARIABLE = strip(scan(Analysis_Variables,I,",")) ;
               put '                <arm:AnalysisVariable ItemOID="IT.' Dataset + (-1) '.' VARIABLE + (-1) '"/>' ;
            end ;
         end ;
         put '              </arm:AnalysisDataset>' ;
      end ;

      if last.Analysis_ID then do ;
         put '            </arm:AnalysisDatasets>' ;
         put '            <arm:Documentation>' ;
         put '              <Description>' ;
         put '                <TranslatedText xml:lang="en">' Documentation + (-1) '</TranslatedText>' ;
         put '              </Description>' ;
         if DocumentRef ne "" then do ;
            put '              <def:DocumentRef leafID="LF.' DocumentRef + (-1) '">' ;
            put '              </def:DocumentRef>' ;
         end ;
         put '            </arm:Documentation>' ;
         if ProgrammingCode ne "" then do ;
            put '            <arm:ProgrammingCode Context="' Version + (-1) '">' ;
            put '              <arm:Code>' ;
            put ProgrammingCode ;
            put '              </arm:Code>' ;
            put '            </arm:ProgrammingCode>' ;
         end ;
         else if ProgramID ne "" then do ;
            put '            <arm:ProgrammingCode Context="' Version + (-1) '">' ;
            put '              <def:DocumentRef leafID="LF.' ProgramID + (-1) '"/>' ;
            put '            </arm:ProgrammingCode>' ;
         end ;
         put '          </arm:AnalysisResult>' ;
      end ;

      if last.ID then do ;
         put '        </arm:ResultDisplay>' ; 
      end ;

      if _EOF then do ;
         put '      </arm:AnalysisResultDisplays>' ;
      end ;
   run ;
%mend _CR_ARM ;

%_CR_ARM ;

